const mongoose = require('mongoose');
const timestamps = require('mongoose-timestamps');
const moment = require('moment')
const numeral = require('numeral')
const Schema = mongoose.Schema;
const PriceTier = require('./priceTier');

const NotNullValidator = {
  validator: function(value) {
    return value !== null
  },
  message: props => `${props.path} cannot be a null value!`
}

const BoxSchema = new Schema({
  breakdown: [{
    distName: String,
    distValue: Number
  }],
  supplierLotId: {
    type: String,
    trim: true,
    validate: NotNullValidator
  },
  supplierDescription: {
    type: String,
    trim: true
  },
  comments: {
    type: String,
    trim: true,
    validate: NotNullValidator
  },
  picture: {
    type: String,
    trim: true,
    validate: NotNullValidator
  },
  attachment: {
    type: String,
    trim: true,
    validate: NotNullValidator
  },
  cycle: {
    type: Date,
  },
  cycleString: {
    type: String
  },
  owner: {
    type: Schema.ObjectId,
    ref: 'Supplier'
  },
  event: {
    type: Schema.ObjectId,
    ref: 'Event'
  },
  type: {
    type: Schema.ObjectId,
    ref: 'BoxType',
    required: true
  },
  creditTerms: {
    type: String,
    enum: ['Cash', '30 Days', '60 Days', '90 Days', '120 Days'],
    validate: NotNullValidator
  },
  bids: [{
    type: Schema.ObjectId,
    ref: 'Bid'
  }],
  looking: [{
    type: Schema.ObjectId,
    ref: 'User'
  }],
  lotId: {
    type: String,
    trim: true
  },
  notifyOnSms: {
    type: Boolean,
    default: false
  },
  notifyFromPrice: Number,
  autoSold: Number,
  sold: {
    type: Boolean,
    default: false
  },
  paid: {
    type: Boolean,
    default: false
  },
  delivered: {
    type: Boolean,
    default: false
  },
  sealed: {
    type: Boolean
  },
  withdrawn: {
    type: Boolean,
    default: false
  },
  initialPrice: Number,
  value: {
    type: Number,
    required: true
  },
  priceTier: {
    type: Schema.ObjectId,
    ref: 'PriceTier'
  },
  cts: {
    type: Number,
    required: true
  },
  ppc: {
    type: Number,
    required: true
  },
  finalPrice: Number
});

BoxSchema.plugin(timestamps);

BoxSchema.pre('validate', function (next) {
  const box = this;

  const proceed = () => {
    if (box.isModified('initialPrice')) {
      box.initialPrice = parseFloat(box.initialPrice).toFixed(2);
    }
  
    if (box.cycle) {
      box.cycleString = moment(box.cycle).format('MMM YY')
    }
  
    return next()
  }

  if (box.isModified('value')) {
    PriceTier.findOne()
      .where('from').lte(box.value)
      .where('to').gte(box.value)
      .exec().then(async tier => {
        if (!tier) {
          const minTier = await PriceTier.findOne({}).sort('from').exec();
          const maxTier = await PriceTier.findOne({}).sort('-to').exec();

          const minPrice = numeral(minTier.from).format('0,0');
          const maxPrice = numeral(maxTier.to).format('0,0');
          return next(`Total price must be between $${minPrice} to $${maxPrice}`);
        }

        box.priceTier = tier._id;
        proceed();
      });
  } else {
    proceed()
  }
});

module.exports = mongoose.model('Box', BoxSchema);
